VERSION 8000; ; 9114
* -----------------------------------------------------------------------------
* @filedoc
* @file Topo_TextPunktBezug.CSX
* @brief Punktreferenzen fr Texte generieren
* 
* Dieses Beispielskript generiert Punktreferenzen fr vorhandene Texte.
* Es werden markierte Texte mit Kode 9999 der Schicht "Text" ausgewertet.  
* Es werden Punkte bercksichtigt, die innerhalb eines Suchkreises um die Koordinate des Textes liegen.
* Hierbei lsst sich die Punktsuche ber die Kodierungen und Markierung einschrnken.
*  - Wird ein vorh. Punkt gefunden, so wird dieser als Textbezugspunkt bernommen.
*  - Wird kein vorh. Punkt gefunden, so wird ein neuer Punkt erzeugt und als Textbezugspunkt bernommen (Punktkode = Textkode).
* Fr die Punktsuche lassen sich auch nur vorhandene Punkte bercksichtigen. 
* 
* -----------------------------------------------------------------------------
* Copyright (C) IB&T
* 
* Gewhrleistung und Nutzung
* 
* Die Nutzung des Skriptes erfolgt stets auf eigene Gefahr. IB&T bernimmt keine
* Gewhr dafr, dass die Nutzung dieses Skriptes zu Ihrem gewnschten Ergebnis
* fhrt. Da CardScripte vorhandene Projektdaten durch Direktzugriff auf die
* jeweiligen Datenbanken auch ohne weitere Sicherheitsabfragen unbemerkt verndern
* oder lschen knnen, erfordert eine Nutzung entsprechende Kenntnisse und groe
* Sorgfalt. IB&T haftet nicht fr eventuell entstehende Datenverluste und
* Folgeschden, die durch die Nutzung von CardScripten entstehen.
*
* -----------------------------------------------------------------------------
* @author JH/CK
* @endfiledoc
* -----------------------------------------------------------------------------
*    8300 18.05.09 JH
*    9144 07.04.21 NR nummerische in alphanummerische Werte umgewandelt (Punkt- und Textkodes)
* @history
* -----------------------------------------------------------------------------


* Vereinbarungen --------------------------------------------------------------

BILDSCHIRM GRAFIK              | Meldungen im Arbeitsprotokoll ausgeben
$PRFSTUFE 7

* Implizite Deklarationen -----------------------------------------------------

* Schicht und Kodebereich fr die auszuwertenden Texte
SYMBOLE * !Schicht = "Text"              | Name der Schicht
SYMBOLE * !von_TKode = "9999"            | Kodebereich 
SYMBOLE * !bis_TKode = "9999"

* Filter fr die zu suchenden Punkte
SYMBOLE * P_Mark = 1                     | Schalter fr: 0 = alle Punkte, 1 = nur markierte Punkte
SYMBOLE * !von_PKode = "0"               | Kodebereich 
SYMBOLE * !bis_PKode = "zzzz"
SYMBOLE * SuchR = 0,2                    | Suchradius (MinMax-Bereich) um Textkoordinate 

SYMBOLE * P_Neu = 1                      | Schalter fr Neupunkte erzeugen: 1 = ja, 0 = nein
SYMBOLE * !PNum_Konst = "PTXT."          | Konstante fr Punktnummer der neu generierten Punkte
SYMBOLE * I;!TEXT;!TKODE;X;Y;!FINDPKT;!PNAME1;!PNAME2;PI;!NR

* Explizite Deklarationen -----------------------------------------------------

* Systemobjekte
SYMBOLE /POINTSYS/ PointSys    | Symbol fr das System der Punkte
SYMBOLE /TOPOSYS/ TopoSys      | Symbol fr das System der Topografiedaten

* Topografieojekte
SYMBOLE /TOPOLAYER/ Schicht    | Symbol fr die Schicht
SYMBOLE /TOPOTEXT/ Text        | Symbol fr den Text
SYMBOLE /TOPOTEXTINDEX/ TextI  | Symbol fr Textindex 

* Punktobjekte
SYMBOLE /POINT/ Point; TPoint  | Symbole fr Punkte
SYMBOLE /POINTINDEX/ PointI    | Symbol fr Punktindex

* Hilfsobjekte
SYMBOLE /COORDINATE/ PKoor     | Symbol fr den Hilfspunkt
SYMBOLE /MINMAX/ MinMax1       | Symbol fr Objektausdehnung (Suchbereich)


* Ausfhrungen ----------------------------------------------------------------

* Ausgeben der Vorgaben/Einstellungen im Protokoll 
MELDE "Einstellungen fr die Punktsuche vorhandener Texte:"
MELDE " "
MELDE "  Schicht     #                "; !Schicht     
MELDE "  Textkode    von #### bis ####"; !von_TKode; !bis_TKode  
MELDE "  Punktkode   von #### bis ####"; !von_PKode; !bis_PKode
MELDE "  Suchradius  #.#              "; SuchR

WENN P_NEU == 1 DANN
  MELDE "  Existiert kein vorhandener Punkt, wird ein neuer Punkt mit der Nummernkonstante '#' generiert (Punktkode = Textkode)."; !PNum_Konst
SONST
  MELDE "  Fr die Punktsuche werden nur vorhandene Punkte bercksichtigt." 
WEITER

MELDE " "
MELDE "Liste der vorhandenen Texte:"
MELDE " "

* Systemobjekte
TopoSys = Project.GetTopoSys()
PointSys = Project.GetPointSys()

* Hilfsobjekt (MinMax-Bereich) 
MinMax1 = NEWOBJECT /MINMAX/

* Schichtobjekt 
Schicht = TopoSys.GetLayerByName(!Schicht) 
  * prft, ob Schicht vorhanden ist
  WENN Schicht == $NULL DANN STOP MELDE "Schicht '#' ist nicht vorhanden --> Punktsuche wird abgebrochen!"; !Schicht

* Textindexobjekt: Auswahl der Texte ber Schicht, Kodebereich und Markierung
TextI = TopoSys.GetTextIndex(!Schicht)                       | liefert Textindexobjekt der Schicht
TextI.SetAlphaCodeFilter(!von_TKode; !bis_TKode)             | setzt einen Filter fr den Textkode
TextI.SetMarkFilter(P_Mark)                                  | setzt einen Filter fr die Textmarkierung

*** Schleife zum Auslesen der Texte ber gesetzte Filter 
I = 0                                                        | Schleifenzhler

* Textobjekt
SOLANGE Text = TextI.GetNext()                               | liest Texte ber gesetzte Filter aus 

  I = I + 1
  !Text = Text.GetText()                                     | liefert den Text
  !TKode = Text.GetAlphaCode()                               | liefert den Textkode
  MELDE "  Nr. #: Punktsuche fr Text '#'"; I; !Text
  
  * Hilfspunktobjekt "PKoor": Koordinate des Textbezugspunktes / der Textposition
  PKoor = Text.GetReferencePosition()                        | liefert die Koordinate des Textbezugspunktes
  WENN PKoor == $NULL DANN PKoor = Text.GetTextPosition()    | wenn Bezugspunkt nicht vorhanden: liefert die Koordinate der Textposition
  x = PKoor.GetX()                                           | liefert den Rechtswert
  y = PKoor.GetY()                                           | liefert den Hochwert
  MELDE "      Textkoordinate #.###  #.###"; x; y
  
  * Hilfsobjekt MinMax-Bereich: Suchkreis um Textkoordinate  
  MinMax1.Reset()
  MinMax1.Add(PKoor.GetX()-SuchR; PKoor.GetY()-SuchR)
  MinMax1.Add(PKoor.GetX()+SuchR; PKoor.GetY()+SuchR)
  *  MinMax1.Draw(7)                                          | zeichnet den Suchbereich (nur fr Kontrolle einschalten)

  * Punktobjekt "TPoint": Punkt des Textbezugspunktes
  TPoint = Text.GetReferencePoint()                          | liefert den vorhandenen Punkt des Textbezugspunktes
  
  WENN TPoint == $NULL DANN                                  | wenn Punktbezug nicht vorhanden: sucht Punkt in Punktdatenbank ber Punktindex (PointI) 
  
    MELDE "      Vorh. Punkt als Textbezugspunkt ist nicht vorhanden, Punkt wird gesucht ..."
    
    * Punktindexobjekt: Auswahl der Punkte innerhalb des Suchradius ber Kodebereich und Markierung
    PointI = PointSys.GetPointIndex(1)
    PointI.SetMinMaxFilter(MinMax1)                          | setzt einen Filter fr den Suchbereich
    PointI.SetAlphaCodeFilter(!von_PKode; !bis_PKode)        | setzt einen Filter fr den Punktkode
    PointI.SetMarkFilter(P_Mark)                             | setzt einen Filter fr die Punktmarkierung
    
    *** Schleife zum Auslesen der Punkte ber gesetzte Filter
    * Punktobjekt "Point"
    SOLANGE Point = PointI.GetNext()                         | liest die Punkte ber gesetzte Filter aus 
      !FindPkt = Point.GetName()                             | liefert die Punktnummer
      Text.SetReferencePoint(Point)                          | bernimmt den Punkt als Textbezugspunkt
      MELDE "      Punkt # als Textbezugspunkt bernommen"; !FindPkt
    ENDE SOLANGE
    *** Ende Schleife
    
    TPoint = Text.GetReferencePoint()                        | prft, ob Text einen vorhandenen Punkt referenziert
    
    WENN TPoint == $NULL DANN                                | wenn Punkt nicht gefunden: erzeugt einen neuen Punkt (sofern der Schalter gesetzt ist)
    
      WENN P_Neu == 1 DANN
      
        MELDE "      Punkt im Suchbereich ist nicht vorhanden, Punkt wird erzeugt ..."
        TEXTGEN !PName1; "######"; I                         | generiert die Nummer fr neuen Punkt
        !PName1 = !PName1 @ "000000"
        !PName2 = !PNum_Konst + !PName1
        
        *** Schleife zum Prfen der Punktnummer: Wenn Punktnummer vorhanden, dann nchste Nummer generieren
        SCHLEIFE  PI = 1; 999999
          TPoint = PointSys.GetPoint(!PName2)
          WENN TPoint == $NULL DANN ABBRUCH SCHLEIFE
          I = I + 1
          TEXTGEN !PName1; "######"; I
          !PName1 = !PName1 @ "000000"
          !PName2 = !PNum_Konst + !PName1
        ENDE SCHLEIFE
        *** Ende Schleife
        
        TPoint = PointSys.CreatePoint(!PName2; 0)            | erzeugt neuen Punkt mit generierter Nummer 
        * prft, ob freie Punktnummer gefunden wird
        WENN TPoint == $NULL DANN STOP MELDE "      Keine freie Punktnummer gefunden --> Punktgenerierung wird abgebrochen!"
        TPoint.SetAlphaCode(!TKode)                          | setzt den Punktkode
        TPoint.SetCoordinate(PKoor)                          | bernimmt die Textkoordinate als Punktkoordinate
        Text.SetReferencePoint(TPoint)                       | bernimmt den vorhandenen Punkt als Textbezugspunkt 
        MELDE "      Punkt # neu erzeugt und als Textbezugspunkt bernommen"; !PName2
        
      WEITER
      
    WEITER

  SONST                                                      | wenn Punktbezug schon vorhanden: Meldung ausgeben
  
    !Nr = TPoint.GetName()
    MELDE "      Textbezug auf Punkt # ist schon vorhanden"; !Nr
    
  WEITER 

  TPoint = $NULL
  
ENDE SOLANGE